package com.winhc.bigdata.task.configuration;

import com.winhc.bigdata.task.service.DingTalkService;
import com.winhc.bigdata.task.util.ThrowableUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.frameworkset.elasticsearch.ElasticSearchHelper;
import org.frameworkset.elasticsearch.boot.BBossESStarter;
import org.frameworkset.elasticsearch.bulk.BulkCommand;
import org.frameworkset.elasticsearch.bulk.BulkInterceptor;
import org.frameworkset.elasticsearch.bulk.BulkProcessorBuilder;
import org.frameworkset.elasticsearch.client.ClientInterface;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.frameworkset.elasticsearch.bulk.BulkProcessor;

/**
 * @author ZhangJi
 * @since 2021-08-03 09:36
 */
@Configuration
@Slf4j
@RequiredArgsConstructor
public class BBossESConfiguration {
    private final DingTalkService dingTalkService;

    @Bean
    public ClientInterface bbossESClient(BBossESStarter bBossESStarter) {
        return bBossESStarter.getConfigRestClient("esmapper/scroll.xml");
    }

    @Bean
    public BulkProcessor buildBulkProcessor() {
        //定义BulkProcessor批处理组件构建器
        BulkProcessorBuilder bulkProcessorBuilder = new BulkProcessorBuilder();
        bulkProcessorBuilder.setBlockedWaitTimeout(0)//指定bulk数据缓冲队列已满时后续添加的bulk数据排队等待时间，如果超过指定的时候数据将被拒绝处理，单位：毫秒，默认为0，不拒绝并一直等待成功为止
                .setBulkFailRetry(1)//如果处理失败，重试次数，暂时不起作用
                .setBulkSizes(1000)//按批处理数据记录数
                .setFlushInterval(5000)//强制bulk操作时间，单位毫秒，如果自上次bulk操作flushInterval毫秒后，数据量没有满足BulkSizes对应的记录数，但是有记录，那么强制进行bulk处理
                .setWarnMultsRejects(1000)//bulk处理操作被每被拒绝WarnMultsRejects次（1000次），在日志文件中输出拒绝告警信息
                .setWorkThreads(20)//bulk处理工作线程数
                .setWorkThreadQueue(20)//bulk处理工作线程池缓冲队列大小
                .setBulkProcessorName("dynamic_processor")//工作线程名称，实际名称为BulkProcessorName-+线程编号
                .setBulkRejectMessage("Reject dynamic bulk processor")//bulk处理操作被每被拒绝WarnMultsRejects次（1000次），在日志文件中输出拒绝告警信息提示前缀
//				.setElasticsearch("default")//指定Elasticsearch集群数据源名称，bboss可以支持多数据源
                .addBulkInterceptor(new BulkInterceptor() {
                    public void beforeBulk(BulkCommand bulkCommand) {

                    }

                    public void afterBulk(BulkCommand bulkCommand, String result) {

                    }

                    public void exceptionBulk(BulkCommand bulkCommand, Throwable exception) {
                        dingTalkService.error("{}", exception.getMessage());
                        exception.printStackTrace();
                    }

                    public void errorBulk(BulkCommand bulkCommand, String result) {
                        log.error("insert some error {}", result);
                    }
                });

        return bulkProcessorBuilder.build();//构建批处理作业组件
    }
}
