package com.winhc.bigdata.task.framework.odps.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.bigdata.task.enums.DataWorksTaskFlowEnum;
import com.winhc.bigdata.task.enums.TaskStatusEnum;
import com.winhc.bigdata.task.framework.odps.entity.PullRequestRecord;
import com.winhc.bigdata.task.framework.odps.mapper.PullRequestRecordMapper;
import com.winhc.bigdata.task.framework.odps.service.OdpsSqlInfoService;
import com.winhc.bigdata.task.framework.odps.service.PullRequestRecordService;
import com.winhc.bigdata.task.service.DataWorksService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/7/6 10:09
 */
@Slf4j
@Service
@AllArgsConstructor
public class PullRequestRecordServiceImpl extends ServiceImpl<PullRequestRecordMapper, PullRequestRecord> implements PullRequestRecordService {
    private final OdpsSqlInfoService odpsSqlInfoService;
    private final DataWorksService dataWorksService;

    @Override
    public synchronized Object pull(String sqlId, String toMongoDbName, String pullBy) {
        Assert.isTrue(odpsSqlInfoService.check(sqlId), "sql id 不存在！请先注册函数 ");
        List<PullRequestRecord> list = lambdaQuery()
                .eq(PullRequestRecord::getSqlId, sqlId)
                .ne(PullRequestRecord::getPartitionTaskStatus, TaskStatusEnum.导出完毕)
                .list();
        if (list.size() >= 1) {
            return "已经有任务在导出中... pull id:" + list.stream().map(PullRequestRecord::getId).map(r -> r + "").collect(Collectors.joining(","));
        }
        PullRequestRecord build = PullRequestRecord.builder()
                .toMongoDbName(toMongoDbName)
                .sqlId(sqlId)
                .pullBy(pullBy)
                .partitionTaskStatus(TaskStatusEnum.等待中)
                .build();
        save(build);
        return build.getId();
    }

    private static final Map<String, DataWorksTaskFlowEnum> map = new HashMap<String, DataWorksTaskFlowEnum>() {
        {
            put("to_mongo", DataWorksTaskFlowEnum.NOT_RUN);
        }
    };

    @Override
    public Object status(String pullId) {
        PullRequestRecord byId = getById(pullId);
        Assert.notNull(byId, "拉取任务的id不存在！");
        String pullInstanceId = byId.getPullInstanceId();
        if (StringUtils.isEmpty(pullInstanceId)) {
            return map;
        }
        return dataWorksService.query("winhc_ng", Long.parseLong(pullInstanceId));

    }
}
