package com.winhc.bigdata.task.util;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import lombok.experimental.ExtensionMethod;
import org.elasticsearch.common.collect.Tuple;
import org.frameworkset.util.Assert;
import org.frameworkset.util.CollectionUtils;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ZhangJi
 * @since 2021-08-03 09:09
 */
@ExtensionMethod(ElasticsearchQueryUtil.QueryUtilExt.class)
public class ElasticsearchQueryUtil {
    public static String updatePersonId(String oldKeyno, String newKeyno) {
        return "{\n" +
                "  \"query\": {\n" +
                "    \"term\": {\n" +
                "      \"association_entity_info.keyno\": {\n" +
                "        \"value\": \"" + oldKeyno + "\"\n" +
                "      }\n" +
                "    }\n" +
                "  },\n" +
                "  \"script\": {\n" +
                "    \"stored\": \"update-person-id\",\n" +
                "    \"params\": {\n" +
                "      \"old_keyno\": \"" + oldKeyno + "\",\n" +
                "\"new_keyno\": \"" + newKeyno + "\"\n" +
                "    }\n" +
                "  }\n" +
                "}";
    }

    public static String deleteDynamic(String rowkey, String tn, List<Tuple<String, String>> creates) {
        return new JSONObject()
                .fluentPut("query", new JSONObject()
                        .fluentPut("bool", new JSONObject()
                                .fluentPut("filter", new JSONArray()
                                        .fluentAdd(
                                                new JSONObject()
                                                        .fluentPut("term", new JSONObject()
                                                                .fluentPut("rowkey", rowkey)
                                                        )
                                        )
                                        .fluentAdd(
                                                new JSONObject()
                                                        .fluentPut("term", new JSONObject()
                                                                .fluentPut("tn", tn)
                                                        )
                                        )
                                )
                        )
                )
                .fluentPut("script", new JSONObject()
                        .fluentPut("stored", "delete-dynamic")
                ).toJSONString();
    }

    public static String createPersonId(String rowkey, String tn, List<Tuple<String, String>> creates) {
        Assert.isTrue(!CollectionUtils.isEmpty(creates), "未找到新增人名");
        return new JSONObject()
                .fluentPut("query", new JSONObject()
                        .fluentPut("bool", new JSONObject()
                                .fluentPut("filter", new JSONArray()
                                        .fluentAdd(
                                                new JSONObject()
                                                        .fluentPut("term", new JSONObject()
                                                                .fluentPut("rowkey", rowkey)
                                                        )
                                        )
                                        .fluentAdd(
                                                new JSONObject()
                                                        .fluentPut("term", new JSONObject()
                                                                .fluentPut("tn", tn)
                                                        )
                                        )
                                )
                        )
                )
                .fluentPut("script", new JSONObject()
                        .fluentPut("stored", "create-person-id")
                        .fluentPut("params", creates.toMap())
                ).toJSONString();

    }

    public static class QueryUtilExt {
        public static Map<String, String> toMap(List<Tuple<String, String>> tuples) {
            return tuples.stream().collect(Collectors.toMap(Tuple::v1, Tuple::v2, (o, o2) -> o));
        }
    }

    public static void main(String[] args) {
        System.out.println(createPersonId("1234", "company", Collections.singletonList(Tuple.tuple("张三", "122333444"))
        ));
    }


}
